Imports iNTERFACEWARE.Chameleon
Imports System.Reflection

' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
' IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FIT-
' NESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
' iNTERFACEWARE BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
' IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CON-
' NECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Public Class ServerModule
    Dim runningAsService As Boolean
    Dim listener As LlpServer
    Dim logStream As System.IO.StreamWriter
    Dim ackEngine As iNTERFACEWARE.Chameleon.DynamicEngine
    Dim nackEngine As iNTERFACEWARE.Chameleon.DynamicEngine

    Const APP_LOG_FILE As String = "hl7log.txt"
    Const APP_LISTEN_PORT As Integer = 5123
    Const APP_MAX_MESSAGE_DISPLAY_LENGTH As Integer = 2048
    Const APP_AUTOACK_VMD As String = "autoack.vmd"
    Const APP_AUTONACK_VMD As String = "autonack.vmd"

    Public Sub New(ByVal isService As Boolean)
        runningAsService = isService
        If (runningAsService) Then
            logStream = System.IO.File.AppendText(ExecutableDirectory() + APP_LOG_FILE)
        End If
    End Sub

    Public Function ExecutableDirectory() As String
        Return System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location) + "\\"
    End Function

    Public Sub StartService()
        Try
            ackEngine = New iNTERFACEWARE.Chameleon.DynamicEngine
            nackEngine = New iNTERFACEWARE.Chameleon.DynamicEngine

            ackEngine.Load(ExecutableDirectory() + APP_AUTOACK_VMD)
            nackEngine.Load(ExecutableDirectory() + APP_AUTONACK_VMD)

            Listener = New LlpServer()
            Log("Starting service.")
            listener.StartListen(APP_LISTEN_PORT)
            Log("Listening on port" + Str(APP_LISTEN_PORT))

            AddHandler listener.ClientConnected, AddressOf Listener_ClientConnected
            AddHandler listener.ClientDisconnected, AddressOf Listener_ClientDisconnected
            AddHandler listener.ClientErrorReceived, AddressOf Listener_ClientErrorReceived
            AddHandler listener.DataIgnored, AddressOf Listener_DataIgnored
            AddHandler listener.ErrorReceived, AddressOf Listener_ErrorReceived
            AddHandler listener.MessageReceived, AddressOf Listener_MessageReceived

            LlpServer.RunMessageLoop()
        Catch ex As Exception
            Log("Error starting service: " + ex.Message)
        End Try
    End Sub

    Public Sub StopService()
        Try
            Log("Stopped service")
            listener.StopMessageLoop()
        Catch ex As Exception
            Log("Error stopping service:" + ex.Message)
        End Try
    End Sub

    Private Function PrettyPrintMessage(ByVal message As String) As String
        Return listener.MakeMessagePrintable(message, Constants.vbCrLf, APP_MAX_MESSAGE_DISPLAY_LENGTH)
    End Function

    Public Sub Log(ByVal line As String)
        Dim logLine As String
        logLine = System.DateTime.Now.ToString() + " " + line
        If (runningAsService) Then
            logStream.WriteLine(logLine)
            logStream.Flush()
        Else
            System.Console.WriteLine(logLine)
        End If
    End Sub

    Private Sub Listener_ClientConnected(ByVal sender As System.Object, ByVal e As System.EventArgs, ByVal client As iNTERFACEWARE.Chameleon.LlpConnection)
        Log("Client connected from " + client.RemoteIpAddressAsString)
    End Sub

    Private Sub Listener_ClientDisconnected(ByVal sender As System.Object, ByVal e As System.EventArgs, ByVal client As iNTERFACEWARE.Chameleon.LlpConnection)
        Log("Client disconnected from " + client.RemoteIpAddressAsString)
    End Sub

    Private Sub Listener_ClientErrorReceived(ByVal sender As System.Object, ByVal e As System.EventArgs, ByVal client As iNTERFACEWARE.Chameleon.LlpConnection, ByVal errorMessage As String, ByVal errorCode As Integer)
        Log("Client error from " + client.RemoteIpAddressAsString + ":" + errorMessage)
    End Sub

    Private Sub Listener_ErrorReceived(ByVal sender As System.Object, ByVal e As System.EventArgs, ByVal errorMessage As String, ByVal code As Integer)
        Log("Server error: " + errorMessage)
    End Sub

    Private Sub Listener_DataIgnored(ByVal sender As System.Object, ByVal e As System.EventArgs, ByVal client As iNTERFACEWARE.Chameleon.LlpConnection, ByVal data As String)
        Log("Ignored data outside of LLP connection from client " + Client.RemoteIpAddressAsString + ":")
        Log(PrettyPrintMessage(data))
    End Sub

    Private Sub Listener_MessageReceived(ByVal sender As System.Object, ByVal e As System.EventArgs, ByVal client As iNTERFACEWARE.Chameleon.LlpConnection, ByVal message As String)
        Dim responseMessage As String

        ' All our networking events from llpServer have a Client object which gives us
        ' information about the host we received a message from.
        Log("Received message from " + client.RemoteIpAddressAsString + ":")
        Log(PrettyPrintMessage(message))

        ' TODO - we don't actually do anything with the message.

        ' The following code generates an auto-acknowledgement or an negative acknowledgement if
        ' the generation of auto-ack fails
        responseMessage = ""
        Try
            responseMessage = ackEngine.GenerateAckFromMessage(message)
        Catch ex As Exception
            Log("Error while generating Ack, sending a Nack instead")
            Try
                responseMessage = nackEngine.GenerateAckFromMessage(message)
            Catch ex2 As Exception
                Log("Error generating ACK" + ex2.Message)
            End Try

        End Try
        If (responseMessage = "") Then
            responseMessage = "MSH|^~\\&|ACME|ACME_APP|XYZ|XYZ_APP|||NACK|XXX|P|2.4|" + ControlChars.Cr + "MSA|AA|XXX|" + ControlChars.Cr
            Log("Error generating Nack, sending a hard-coded Nack instead")
        End If
        Log(PrettyPrintMessage(responseMessage))
        ' Sends the generated ack/nack
        client.SendMessage(responseMessage)
    End Sub

End Class
